# File name: 
__name__ = 'CountMe'
# Copyright: The AustSTEM Foundation Limited
# Author: Tony Strasser
# Date created: 19 August 2018
# Date last modified: 22 November 2020
# Release Number: 1
# MicroPython Version: 1.12 for the Kookaberry V4-05
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation.
# To the fullest extent permitted by law, AustSTEM absolutely disclaims 
# all warranties, expressed or implied, including, but not limited to, 
# implied warranties of merchantability and fitness for any particular purpose. 
# AustSTEM gives no warranty that this software will be free of errors, 
# or that defects in the software will be corrected,  
# See the GNU General Public License for more details.
#
# This is a totaliser program which counts key presses on the Kookaberry buttons
# The totals of each key pressed displayed
# Controls are provided to Exit, Reset, Pause, and Save the totals to a file
# Lesson plan designator added to the radio message.

import machine, kooka, fonts, time
from Kapputils import config    # utility to read the configuration file
import kbtimer    # module to manage timed buttons

lesson = 'KLP001'    # lessonplan designation
btn_ids = ['R','B','Y','G']    # key identities
btn_count = [0] * len(btn_ids)    # totaliser for each key (and no key)
btn_pause = 0    # flag to pause counting
pause = ['Pse','Res']    # display strings for pause control
long_press = 1000    # minimum time for long press button functions
short_press = int(long_press/2)    # maximum time for short press button functions

a_timer = kbtimer.KBTimer(kooka.button_a, long_press) # buttom A timer object
b_timer = kbtimer.KBTimer(kooka.button_b, long_press) # buttom B timer object
c_timer = kbtimer.KBTimer(kooka.button_c, long_press) # buttom C timer object
d_timer = kbtimer.KBTimer(kooka.button_d, long_press) # buttom C timer object

disp = kooka.display
params = config('Kappconfig.txt')   # read the configuration file
chan = int(params['CHANNEL'])      # use channel from the configuration file
baud = int(params['BAUD'])      # use data rate from the configuration file
pwr = int(params['POWER'])      # use transmit power from the configuration file
kooka.radio.enable()
kooka.radio.config(channel=chan, data_rate=baud, power=pwr) # set up the radio
# Construct a 4 character id
id=''
for i in range(0,min(2,len(params['NAME']))): id = id + params['NAME'][i]
for i in range(0,min(2,len(params['SURNAME']))): id = id + params['SURNAME'][i]
# Clear any interim button presses to obviate false counts
if kooka.button_a.was_pressed(): pass
if kooka.button_b.was_pressed(): pass
if kooka.button_c.was_pressed(): pass
if kooka.button_d.was_pressed(): pass

while not a_timer.isexpired(): # long press to exit
    disp.fill(0)
    disp.setfont(fonts.mono8x8)
    disp.text('%s' % __name__, 0, 10)
    disp.setfont(fonts.mono6x7)
    disp.text('%s' % id, 100, 10)
    disp.text('Long press >1sec:',0,50)
    disp.text('X', 6, 63)	    # A Key = Exit
    disp.text('Rst', 24, 63)	    # C Key = Reset totals
    disp.text('%s' % pause[btn_pause], 64, 63)	    # D Key = Pause counting
    disp.text('Sav', 100, 63)	    # B Key = Save results
# Adjust controls by means of Kookaberry keys
    if c_timer.isexpired():
        c_timer.reset() 
        for x in range(0,len(btn_count)): btn_count[x] = 0    # reset totals
    if d_timer.isexpired(): 
        d_timer.reset()
        btn_pause = not btn_pause    # pause counting
        # Clear any interim button presses
        if kooka.button_a.was_pressed(): pass
        if kooka.button_b.was_pressed(): pass
        if kooka.button_c.was_pressed(): pass
        if kooka.button_d.was_pressed(): pass
    if b_timer.isexpired(): 
        b_timer.reset()
        f = open(__name__+'.csv','w+')         # open a text file for writing
        f.write('ID,%s,%s,%s,%s\n' % (btn_ids[0],btn_ids[1],btn_ids[2],btn_ids[3]))   # write the heading line
        f.write('%s,%d,%d,%d,%d\n' % (id,btn_count[0],btn_count[1],btn_count[2],btn_count[3]))   # write the counts
        f.close()    # close file to guard against corruption
        print('%s,%d,%d,%d,%d' % (id,btn_count[0],btn_count[1],btn_count[2],btn_count[3]))
        kooka.radio.send('%s,%s,%d,%d,%d,%d' % (lesson,id,btn_count[0],btn_count[1],btn_count[2],btn_count[3]))

# Count button presses
    if not btn_pause:
        if kooka.button_a.was_pressed():
            time.sleep_ms(short_press)    # wait for release
            if not kooka.button_a.is_pressed(): btn_count[0] += 1
        if kooka.button_c.was_pressed():
            time.sleep_ms(short_press)    # wait for release
            if not kooka.button_c.is_pressed(): btn_count[1] += 1
        if kooka.button_d.was_pressed():
            time.sleep_ms(short_press)    # wait for release
            if not kooka.button_d.is_pressed(): btn_count[2] += 1
        if kooka.button_b.was_pressed():
            time.sleep_ms(short_press)    # wait for release
            if not kooka.button_b.is_pressed(): btn_count[3] += 1

    disp.setfont(fonts.mono8x13)
    for x in range(0,len(btn_ids)):
        disp.text('%s' % (btn_ids[x]), (15 + 30 * x), 24)
        disp.text('%d' % (btn_count[x]), (15 + 30 * x), 38)
    disp.show()
#    time.sleep_ms(100)

